"""
Auditor has one task: inspection of metadata.json and PPR.xml for required properties
"""
import logging
import os.path
from pathlib import Path
from typing import List
from bs4 import BeautifulSoup
import json

from casa_envoy.interfaces import AuditorIF
from workspaces.system.schema import AbstractFile


def get_fields_for(filename: str) -> list:
    metadata_list = ["fileSetIds", "workflowName", "systemId", "productLocator", "destinationDirectory"]
    ppr_list = ["RootDirectory", "RelativePath", "SdmIdentifier"]

    if filename == "PPR.xml":
        return ppr_list

    if filename == "metadata.json":
        return metadata_list


def get_value_for(filename:str, key:str) -> str:
    if ".xml" in filename:
        with open(filename) as file:
            ppr_content = BeautifulSoup(file.read(), "xml")
            return ppr_content.find(key).string

    if ".json" in filename:
        with open(filename) as file:
            metadata = json.loads(file.read())
            return metadata[key]


class AuditFiles(AuditorIF):
    def __init__(self, files: List[str]):
        self.files = files
        self.logger = logging.getLogger("vela")

    def read_file(self, filename: str) -> AbstractFile:
        if os.path.isfile(filename):
            with open(filename) as file:
                if ".json" in filename:
                    metadata = json.loads(file.read())
                    return AbstractFile(filename, json.dumps(metadata).encode('ascii'))
                else:
                    if filename == "PPR.xml":
                        ppr = file.read()
                        return AbstractFile(filename, ppr.encode('ascii'))

    def check_required_fields(self, file: AbstractFile, fields: list) -> bool:
        missing = []
        if file.filename == "PPR.xml":
            ppr_content = BeautifulSoup(file.content.decode(), "xml")
            for tag in fields:
                content = ppr_content.find(tag).string
                if not content:
                    missing.append(tag)

        if file.filename == "metadata.json":
            for tag in fields:
                metadata = json.loads(file.content.decode())
                if tag not in metadata or len(metadata[tag]) == 0:
                    missing.append(tag)
        if len(missing) > 0:
            print(f"Missing fields: {missing}")
            return False
        else:
            return True

    def audit(self) -> bool:
        invalid_files = []
        for file in self.files:
            f = self.read_file(file)
            valid = self.check_required_fields(file=f, fields=get_fields_for(f.filename))
            if not valid:
                invalid_files.append(f.filename)

        if len(invalid_files) != 0:
            self.logger.info(f"INVALID FILE FOUND: {invalid_files}")
            return False
        else:
            return True


class AuditDirectories(AuditorIF):
    def __init__(self):
        self.logger = logging.getLogger("vela")
        self.rootDirectory = get_value_for("PPR.xml", "RootDirectory")
        self.relative_path = get_value_for("PPR.xml", "RelativePath")
        self.sdmId = get_value_for("PPR.xml", "SdmIdentifier")

    def audit(self) -> bool:
        current = os.getcwd()
        needed = self.rootDirectory + "/" + self.relative_path
        if needed != current:
            self.logger.info("DIRECTORY ERROR: not in correct directory for processing.")
            return False
        else:
            working = Path(current + "/working").is_dir()
            products = Path(current + "/products").is_dir()
            rawdata = Path(current + "/rawdata").is_dir()

            if working and products and rawdata:
                self.logger.info("Checking for rawdata...")
                data = Path(current + "/rawdata/" + self.sdmId).is_dir()
                if data:
                    self.logger.info("Data is available. Proceeding...")
                    return True
                else:
                    self.logger.info("FAILURE: data not found in rawdata/")
                    return False
