""" A locations report is produced by the archive service; you give
    it a product locator and it returns a dictionary of details on how
    to retrieve the product's files from long term storage (NGAS): this
    class handles fetching the report from the service or reading it from
    a file, and has utilities to manipulate the report.

"""

import http
import logging
from pathlib import Path
from typing import List, Dict, NamedTuple, Union

# pylint: disable=C0116, E0239, E0401, E0402, E1136, R0201, R0801, R0903, W0613, W1203

import requests

from marshmallow import Schema, fields, post_load

from .errors import (
    LocationServiceTimeoutException,
    LocationServiceRedirectsException,
    LocationServiceErrorException,
    NoLocatorException,
)
from .utilities import (
    Location,
    Cluster,
    RetrievalMode,
    LOG_FORMAT,
    get_capo_settings,
    get_file_info_from_json_file,
    LUSTRE,
)

logger = logging.getLogger(__name__)
formatter = logging.Formatter(LOG_FORMAT)
handler = logging.StreamHandler()
handler.setFormatter(fmt=formatter)
logger.addHandler(handler)


class NgasServer(NamedTuple):
    """Encapsulates the "server" section of a "files" item in locations
    report"""

    server: str
    location: Location
    cluster: Cluster


class NgasFile(NamedTuple):
    """Encapsulates all the information about an NGAS file in a locations
    report"""

    ngas_file_id: str
    subdirectory: Path
    relative_path: str
    checksum: int
    checksum_type: str
    size: int
    version: int
    server: NgasServer


class NgasTarget:
    """ A product we're going to retrieve, and from where """

    def __init__(self, ngas_file: NgasFile, exec_site: str):
        self.ngas_file = ngas_file
        self.retrieve_method = self.determine_retrieve_mode(self.ngas_file.server, exec_site)

    @staticmethod
    def determine_retrieve_mode(server: NgasServer, exec_site: str) -> RetrievalMode:
        """
        Given the execution site (from Capo), location, and cluster,
        decide whether to stream the file or to use the direct-copy plugin.

        :return:
        """

        # If we're not executing on /lustre, direct copy will fail.
        current_dir = Path.cwd().absolute()
        if f"/{LUSTRE}" not in str(current_dir):
            return RetrievalMode.STREAM

        location = server.location
        if server.cluster == Cluster.DSOC and location.value == exec_site:
            return RetrievalMode.COPY
        return RetrievalMode.STREAM

    @property
    def ngas_file_id(self):
        return self.ngas_file.ngas_file_id

    @property
    def subdirectory(self):
        return self.ngas_file.subdirectory

    @property
    def relative_path(self):
        return self.ngas_file.relative_path

    @property
    def checksum(self):
        return self.ngas_file.checksum

    @property
    def checksum_type(self):
        return self.ngas_file.checksum_type

    @property
    def size(self):
        return self.ngas_file.size

    @property
    def version(self):
        return self.ngas_file.version

    @property
    def server(self):
        return self.ngas_file.server


class LocationsReport:
    """ Data from a full locations report such that marshmallow can load it """

    def __init__(self, files: List[NgasFile], aggregate_size: int):

        df_settings = get_capo_settings()
        exec_site = df_settings["execution_site"]
        self.targets = []
        for ngas_file in files:
            self.targets.append(NgasTarget(ngas_file, exec_site))
        self.aggregate_size = aggregate_size


class LocationsReportBuilder:
    """ Uses marshmallow to build a locations report for datafetcher. """

    def __init__(self, item: Union[Path, str]):
        if isinstance(item, Path):
            self.product_locator = None
            self.locations_file = item
        else:
            self.product_locator = item
            self.locations_file = None

    def build(self):
        if self.product_locator:
            locations_dict = self.get_locations_from_service()
        else:
            locations_dict = self.get_locations_from_file()
        return LocationsReportSchema().load(locations_dict)

    def get_locations_from_service(self) -> Dict:
        """
        Use 'requests' to fetch the location report JSON from the locator
        service.

        :return: location report as Dict
        """

        df_settings = get_capo_settings()
        url = df_settings["locator_service_url"]
        logger.debug(f"fetching report from {url}\n\tfor {self.product_locator}")

        # this is needed to prevent SSL errors when tests are run
        # inside a Docker container
        requests.packages.urllib3.util.ssl_.DEFAULT_CIPHERS += ":HIGH:!DH:!aNULL"
        requests.Session().mount(url, adapter=requests.adapters.HTTPAdapter())

        try:
            response = requests.get(url, params={"locator": self.product_locator})
        except requests.exceptions.Timeout as exc_to:
            raise LocationServiceTimeoutException() from exc_to
        except requests.exceptions.TooManyRedirects as exc_re:
            raise LocationServiceRedirectsException() from exc_re
        except requests.exceptions.RequestException as ex:
            raise LocationServiceErrorException(ex) from ex

        if response.status_code == http.HTTPStatus.OK:
            return response.json()
        if response.status_code == http.HTTPStatus.NOT_FOUND:
            raise NoLocatorException(f'locator "{self.product_locator}" not found')
        raise LocationServiceErrorException(f"locator service returned {response.status_code}")

    def get_locations_from_file(self) -> Dict:
        return get_file_info_from_json_file(self.locations_file)


class NgasServerSchema(Schema):
    """ marshmallow schema to interpret "server" section of a "files" item """

    server = fields.Str()
    location = fields.Str()
    cluster = fields.Str()

    @post_load
    def make_ngas_server(self, data, **kwargs):
        return NgasServer(data["server"], Location[data["location"]], Cluster[data["cluster"]])


class NgasFileSchema(Schema):
    """ One of the items in a location report's "files" list """

    ngas_file_id = fields.Str()
    subdirectory = fields.Str()
    relative_path = fields.Str()
    checksum = fields.Str()
    checksum_type = fields.Str()
    version = fields.Integer()
    size = fields.Integer()
    server = fields.Nested(NgasServerSchema())

    @post_load
    def make_filespec(self, data, **kwargs):
        return NgasFile(**data)


class LocationsReportSchema(Schema):
    """ Encapsulates an entire locations report """

    files = fields.List(fields.Nested(NgasFileSchema()))
    aggregate_size = fields.Integer()

    @post_load
    def make_report(self, data, **kwargs):
        return LocationsReport(**data)
